﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Globalization;
using System.Diagnostics;

namespace NumericEntryDemo
{
    /// <summary>
    /// Interaction logic for CalcWindow.xaml
    /// </summary>
    public partial class CalcWindow : Window
    {
        private int _initialValue = 0;
        private bool _canceled = false;
        private MathOperation _currentOp = MathOperation.None;

        //value of current input
        public static readonly DependencyProperty CurrentValueProperty =
            DependencyProperty.Register("CurrentValue", typeof(int),
            typeof(CalcWindow), new PropertyMetadata(0));

        //current input in string form
        public static readonly DependencyProperty CurrentInputProperty =
            DependencyProperty.Register("CurrentInput", typeof(string),
            typeof(CalcWindow), new PropertyMetadata(""));

        //previously calculated value
        public static readonly DependencyProperty PreviousValueProperty =
            DependencyProperty.Register("PreviousValue", typeof(int),
            typeof(CalcWindow), new PropertyMetadata(0));

        private static readonly Dictionary<Key, int> _keyDigits =
            new Dictionary<Key, int>();
        private static readonly Dictionary<Key, MathOperation> _keyOps =
            new Dictionary<Key, MathOperation>();

        private readonly Dictionary<Button, int> _buttonDigits =
            new Dictionary<Button, int>();
        private readonly Dictionary<Button, MathOperation> _buttonOps =
            new Dictionary<Button, MathOperation>();

        static CalcWindow()
        {
            _keyDigits[Key.D0] = 0;
            _keyDigits[Key.D1] = 1;
            _keyDigits[Key.D2] = 2;
            _keyDigits[Key.D3] = 3;
            _keyDigits[Key.D4] = 4;
            _keyDigits[Key.D5] = 5;
            _keyDigits[Key.D6] = 6;
            _keyDigits[Key.D7] = 7;
            _keyDigits[Key.D8] = 8;
            _keyDigits[Key.D9] = 9;

            _keyDigits[Key.NumPad0] = 0;
            _keyDigits[Key.NumPad1] = 1;
            _keyDigits[Key.NumPad2] = 2;
            _keyDigits[Key.NumPad3] = 3;
            _keyDigits[Key.NumPad4] = 4;
            _keyDigits[Key.NumPad5] = 5;
            _keyDigits[Key.NumPad6] = 6;
            _keyDigits[Key.NumPad7] = 7;
            _keyDigits[Key.NumPad8] = 8;
            _keyDigits[Key.NumPad9] = 9;

            _keyOps[Key.Add] = MathOperation.Add;
            _keyOps[Key.Subtract] = MathOperation.Subtract;
            _keyOps[Key.Multiply] = MathOperation.Multiply;
            _keyOps[Key.Divide] = MathOperation.Divide;

#if DEBUG
            _keyOps[Key.A] = MathOperation.Add;
            _keyOps[Key.S] = MathOperation.Subtract;
            _keyOps[Key.M] = MathOperation.Multiply;
            _keyOps[Key.D] = MathOperation.Divide;
#endif

        }

        public int CurrentValue
        {
            get
            {
                return (Int32)GetValue(CurrentValueProperty);
            }
            set
            {
                SetValue(CurrentValueProperty, value);
            }
        }

        public string CurrentInput
        {
            get
            {
                return GetValue(CurrentInputProperty) as string;
            }
            set
            {
                SetValue(CurrentInputProperty, value);
            }
        }

        public int PreviousValue
        {
            get
            {
                return (Int32)GetValue(PreviousValueProperty);
            }
            set
            {
                SetValue(PreviousValueProperty, value);
            }
        }

        public CalcWindow(int initialValue, MathOperation operation)
        {
            _initialValue = initialValue;
            _currentOp = operation;

            CurrentValue = _initialValue;
            PreviousValue = CurrentValue;

            InitializeComponent();

            InitializeButtons();
        }

        #region Generic Input Handling
        private bool HandleEscape()
        {
            _canceled = true;
            CurrentValue = _initialValue;

            Close();

            return true;
        }

        private bool HandleClear()
        {
            CurrentInput = "";
            CurrentValue = 0;

            return true;
        }

        private bool HandleEquals()
        {
            //The final operation will be 
            //handled when the window closes
            Close();
            return true;
        }

        private bool HandleBack()
        {
            if (!string.IsNullOrEmpty(CurrentInput))
            {
                CurrentInput = CurrentInput.Substring(0, CurrentInput.Length - 1);
                //this is guaranteed to succeed because we validate when we add digits
                if (!string.IsNullOrEmpty(CurrentInput))
                {
                    CurrentValue = Int32.Parse(CurrentInput);
                }
                else
                {
                    CurrentValue = 0;
                }
            }
            return true;
        }

        private bool HandleDigit(int digit)
        {
            string newInput = CurrentInput + digit.ToString();
            int temp = 0;
            //make sure the full input is a number
            if (Int32.TryParse(newInput, out temp))
            {
                CurrentInput = newInput;
                CurrentValue = temp;
                return true;
            }
            return false;
        }

        private bool HandleOperation(MathOperation op)
        {
            if (CurrentInput.Length > 0)
            {
                ProcessCurrentOperation();
            }
            _currentOp = op;
            return true;
        }
        
        #endregion

        #region Button Handling

        private void InitializeButtons()
        {
            _buttonDigits[_button0] = 0;
            _buttonDigits[_button1] = 1;
            _buttonDigits[_button2] = 2;
            _buttonDigits[_button3] = 3;
            _buttonDigits[_button4] = 4;
            _buttonDigits[_button5] = 5;
            _buttonDigits[_button6] = 6;
            _buttonDigits[_button7] = 7;
            _buttonDigits[_button8] = 8;
            _buttonDigits[_button9] = 9;

            _buttonOps[_buttonAdd] = MathOperation.Add;
            _buttonOps[_buttonSubtract] = MathOperation.Subtract;
            _buttonOps[_buttonMultiply] = MathOperation.Multiply;
            _buttonOps[_buttonDivide] = MathOperation.Divide;
        }

        void opButton_Click(object sender, RoutedEventArgs e)
        {
            MathOperation op = _buttonOps[sender as Button];
            e.Handled = HandleOperation(op);
        }

        void digitButton_Click(object sender, RoutedEventArgs e)
        {
            int digit = _buttonDigits[sender as Button];
            e.Handled = HandleDigit(digit);
        }

        private void clearButton_click(object sender, RoutedEventArgs e)
        {
            e.Handled = HandleClear();
        }

        private void backButton_Click(object sender, RoutedEventArgs e)
        {
            e.Handled = HandleBack();
        }

        private void equalsButton_Click(object sender, RoutedEventArgs e)
        {
            e.Handled = HandleEquals();
        }
        #endregion

        #region Keyboard Handling
        protected override void OnPreviewKeyDown(KeyEventArgs e)
        {
            /*take over all keyboard input
             * this is important because if a button has focus 
             * and we hit enter then the button will be clicked
             * */

            int digit = 0;
            MathOperation op = MathOperation.None;
            if (_keyDigits.TryGetValue(e.Key, out digit))
            {
                e.Handled = HandleDigit(digit);
            }
            else if (_keyOps.TryGetValue(e.Key, out op))
            {
                e.Handled = HandleOperation(op);
            }
            else
            {
                switch (e.Key)
                {
                    case Key.Enter:
                        e.Handled = HandleEquals();
                        break;
                    case Key.Escape:
                        e.Handled = HandleEscape();
                        break;
                    case Key.Back:
                        e.Handled = HandleBack();
                        break;
                    case Key.C:
                        e.Handled = HandleClear();
                        break;
                    default:
                        e.Handled = false;
                        break;
                }
            }
            base.OnPreviewKeyDown(e);
        }

        #endregion

        private void ProcessCurrentOperation()
        {
            if (string.IsNullOrEmpty(CurrentInput))
            {
                return;
            }
            
            int newValue = 0;
            checked
            {
                try
                {
                    switch (_currentOp)
                    {
                        case MathOperation.Add:
                            newValue = PreviousValue + CurrentValue;
                            break;
                        case MathOperation.Subtract:
                            newValue = PreviousValue - CurrentValue;
                            break;
                        case MathOperation.Multiply:
                            newValue = PreviousValue * CurrentValue;
                            break;
                        case MathOperation.Divide:
                            if (CurrentValue != 0)
                            {
                                newValue = PreviousValue / CurrentValue;
                            }
                            else
                            {
                                newValue = CurrentValue;
                            }
                            break;
                        default:
                            Debug.Assert(false, "Invalid operation. Should never happen!");
                            break;
                    }

                    CurrentInput = "";
                    CurrentValue = newValue;
                    PreviousValue = newValue;
                }
                catch (OverflowException)
                {
                    //will cause the window to close, so we better cancel everything first
                    _canceled = true;
                    CurrentValue = _initialValue;

                    MessageBox.Show("The result overflowed, calculation canceled.");
                }
            }
        }

        protected override void OnDeactivated(EventArgs e)
        {
            if (!_canceled)
            {
                ProcessCurrentOperation();
                if (IsVisible)
                {
                    //window is still visible if we've 
                    //clicked away from the window
                    //before finishing calculation
                    Close();
                }
            }
            base.OnDeactivated(e);
        }
    }
}
